/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.adapters

import android.content.Intent
import android.net.Uri
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.jbo.vga.R
import com.jbo.vga.model.Credit
import kotlinx.android.synthetic.main.view_holder_credits.view.*

class CreditsRecyclerViewAdapter(private var credits: List<Credit>?) : RecyclerView.Adapter<CreditsRecyclerViewAdapter.ViewHolder>() {

    fun setCredits(creditsList: List<Credit>) {
        credits = creditsList
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view: View = LayoutInflater.from(parent.context).inflate(R.layout.view_holder_credits, parent, false)
        return ViewHolder(view)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        credits.let {
            holder.credit = it?.get(position)
            holder.view.tvTitle.text = it?.get(position)?.title ?: ""
            holder.view.tag = it?.get(position)?.title ?: ""
        }
    }

    override fun getItemCount(): Int {
        return credits?.size ?: 0
    }

    inner class ViewHolder(val view: View) : RecyclerView.ViewHolder(view) {
        var credit: Credit? = null

        init {
            view.tvLicense.setOnClickListener { v ->
                val context = v.context
                val intent = Intent(Intent.ACTION_VIEW)
                intent.data = Uri.parse(credit?.link)
                context.startActivity(intent)
            }
        }

        override fun toString(): String {
            return super.toString() + " '" + view.tvTitle.text + "'"
        }
    }

}