/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.adapters

import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.RecyclerView
import com.jbo.vga.R
import com.jbo.vga.didacticunit.DidacticUnitActivity
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.extensions.levelDarkColor
import com.jbo.vga.model.DidacticUnit
import kotlinx.android.synthetic.main.view_holder_index_didactic_unit.view.*

class IndexRecyclerViewAdapter(private val level: String, private var didacticUnits: List<DidacticUnit>?) : RecyclerView.Adapter<IndexRecyclerViewAdapter.ViewHolder>() {

    private var canStartDidacticUnit = true

    fun setCanStartDidacticUnit(can: Boolean){
        canStartDidacticUnit = can
    }

    fun setIndex(index: List<DidacticUnit>) {
        didacticUnits = index
        notifyDataSetChanged()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view: View = LayoutInflater.from(parent.context).inflate(R.layout.view_holder_index_didactic_unit, parent, false)
        return ViewHolder(view)
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        didacticUnits.let {
            holder.didacticUnit = it?.get(position)
            holder.view.tvOrder.text = (holder.adapterPosition + 1).toString()
            holder.view.tvTitleDe.text = it?.get(position)?.titleDE ?: ""
            holder.view.tvTitleEs.text = it?.get(position)?.titleES ?: ""
            holder.view.tvStatusMessage.text = it?.get(position)?.statusMessage ?: ""

            if (holder.didacticUnit?.status == DIDACTIC_UNIT_STATUS_COMING_SOON) {
                holder.view.isEnabled = false
                holder.view.tvStatusMessage.visibility = View.VISIBLE
                holder.view.cvDidacticUnit.setCardBackgroundColor(ContextCompat.getColor(holder.view.context, R.color.didacticUnitStatusBackground))
            }

            holder.view.tag = it?.get(position)?.titleDE ?: ""
        }
    }

    override fun getItemCount(): Int {
        return didacticUnits?.size ?: 0
    }

    inner class ViewHolder(val view: View) : RecyclerView.ViewHolder(view) {
        var didacticUnit: DidacticUnit? = null

        init {
            view.tvOrder.setTextColor(ContextCompat.getColor(view.context, levelColor(level)))
            view.tvStatusMessage.setTextColor(ContextCompat.getColor(view.context, levelDarkColor(level)))
            view.setOnClickListener { v ->
                if (canStartDidacticUnit) {
                    canStartDidacticUnit = false
                    val context = v.context
                    val intent = Intent(context, DidacticUnitActivity::class.java)
                    intent.putExtra(DidacticUnitActivity.ARG_LEVEL, level)
                    intent.putExtra(DidacticUnitActivity.ARG_DIDACTIC_UNIT_NUMBER, adapterPosition)
                    intent.putExtra(DidacticUnitActivity.ARG_DIDACTIC_UNIT_REQUIRES_RESOURCES, didacticUnit?.requiresResources.toBoolean())
                    context.startActivity(intent)
                }
            }
        }

        override fun toString(): String {
            return super.toString() + " '" + view.tvTitleDe.text + "'"
        }
    }

    companion object {
        const val DIDACTIC_UNIT_STATUS_COMING_SOON = "coming_soon"
    }
}