/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.credits

import android.app.Dialog
import android.os.Build
import android.os.Bundle
import android.view.MenuItem
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.recyclerview.widget.DividerItemDecoration
import androidx.recyclerview.widget.LinearLayoutManager
import com.jbo.vga.R
import com.jbo.vga.adapters.CreditsRecyclerViewAdapter
import com.jbo.vga.extensions.setProgressDialog
import com.jbo.vga.model.Credit
import com.jbo.vga.model.RemoteDataSource
import kotlinx.android.synthetic.main.activity_credits.*
import java.util.*

class CreditsActivity : AppCompatActivity(), CreditsContract.ViewInterface {

    private lateinit var creditsPresenter: CreditsContract.PresenterInterface
    private lateinit var dialog: Dialog
    private var adapter: CreditsRecyclerViewAdapter? = null

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_credits)

        initStatusBar()

        initToolbar()

        initPresenter()

        initRecyclerView()

        dialog = setProgressDialog(this, R.color.colorPrimary, getString(R.string.loading_credits))

        creditsPresenter.getCredits()
    }

    private fun initStatusBar() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            window.statusBarColor = ContextCompat.getColor(this, R.color.colorPrimaryDark)
        }
    }

    private fun initToolbar() {
        setSupportActionBar(tbCredits)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }

    private fun initPresenter(){
        val remoteDataSource = RemoteDataSource()
        creditsPresenter = CreditsPresenter(this, remoteDataSource)
    }

    private fun initRecyclerView(){
        rvCredits.addItemDecoration(DividerItemDecoration(this, LinearLayoutManager.VERTICAL))
        val layoutManager = LinearLayoutManager(this)
        rvCredits.layoutManager = layoutManager
        adapter = CreditsRecyclerViewAdapter(ArrayList())
        rvCredits.adapter = adapter
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
            else -> false
        }
    }

    override fun displayCredits(credits: List<Credit>){
        adapter?.setCredits(credits)
    }

    override fun showProgress() {
        dialog.show()
    }

    override fun hideProgress() {
        dialog.dismiss()
    }

    override fun displayError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
    }

    override fun onStop() {
        super.onStop()
        creditsPresenter.stop()
    }
}