/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.credits

import com.jbo.vga.model.Credit
import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.annotations.NonNull
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.observers.DisposableObserver
import io.reactivex.schedulers.Schedulers

class CreditsPresenter(private var viewInterface: CreditsContract.ViewInterface, private var remoteDataSource: RemoteDataSource)  : CreditsContract.PresenterInterface {

    private val compositeDisposable = CompositeDisposable()

    /// CALL RESULTS ///
    private val creditsObservable: Observable<List<Credit>> get() = remoteDataSource.getCredits()
    private val creditsObserver: DisposableObserver<List<Credit>>
        get() = object : DisposableObserver<List<Credit>>() {

            override fun onNext(credits: List<Credit>) {
                viewInterface.hideProgress()
                viewInterface.displayCredits(credits)
            }

            override fun onError(@NonNull e: Throwable) {
                viewInterface.hideProgress()
                viewInterface.displayError(e.message.toString())
            }

            override fun onComplete() {
                //
            }
        }

    /// CALLS ///
    override fun getCredits() {

        val creditsDisposable = creditsObservable
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(creditsObserver)

        compositeDisposable.add(creditsDisposable)
        viewInterface.showProgress()
    }

    override fun stop() {
        compositeDisposable.clear()
    }

}