package com.jbo.vga.extensions

import android.app.Activity
import android.content.Context
import android.content.Context.INPUT_METHOD_SERVICE
import android.content.Intent
import android.content.res.ColorStateList
import android.graphics.Color
import android.net.Uri
import android.os.Build
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.view.inputmethod.InputMethodManager
import android.widget.LinearLayout
import android.widget.ProgressBar
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.startActivity
import com.jbo.vga.R
import java.io.File

fun levelColor(level: String): Int {
    return when (level) {
            "a1" -> R.color.colorA1
            "a2" -> R.color.colorA2
            "b1" -> R.color.colorB1
            "b2" -> R.color.colorB2
            else -> R.color.colorPrimary
        }
}

fun levelDarkColor(level: String): Int {
    return when (level) {
        "a1" -> R.color.colorA1Dark
        "a2" -> R.color.colorA2Dark
        "b1" -> R.color.colorB1Dark
        "b2" -> R.color.colorB2Dark
        else -> R.color.colorPrimaryDark
    }
}

fun levelIcon(level: String): Int {
    return when (level) {
        "a1" -> R.drawable.ic_a1
        "a2" -> R.drawable.ic_a2
        "b1" -> R.drawable.ic_b1
        "b2" -> R.drawable.ic_b2
        else -> R.drawable.ic_a1
    }
}

fun createDidacticUnitDirectory(path: String) {
    // Create directory structure if not exists
    val dir = File(path)
    if (!dir.exists())
        dir.mkdirs()
}

fun buildDidacticUnitRemotePath(level: String, didacticUnitNumber: String): String {
    return "levels/$level/didactic-units/$didacticUnitNumber/"
}

fun buildCachePath(cacheDirectory: File, level: String, didacticUnitNumber: String): String {
    return "$cacheDirectory/${buildDidacticUnitRemotePath(level, didacticUnitNumber)}"
}

fun didacticUnitDeleteDownloadCompletedFile(cachePath: String) {
    val file = File("$cachePath/didactic_unit_download_completed")
    if (file.exists()) {
        file.delete()
    }
}

fun didacticUnitDownloadCompletedFile(cachePath: String): File {
    return File("$cachePath/didactic_unit_download_completed")
}

fun didacticUnitDownloaded(cacheDirectory: File, level: String, didacticUnitNumber: String): Boolean {
    val cachePath = buildCachePath(cacheDirectory, level, didacticUnitNumber)
    return didacticUnitDownloadCompletedFile(cachePath).exists()
}

fun setProgressDialog(context: Context, progressColor: Int, message:String): AlertDialog {
    val llPadding = 30
    val ll = LinearLayout(context)
    ll.orientation = LinearLayout.HORIZONTAL
    ll.setPadding(llPadding, llPadding, llPadding, llPadding)
    ll.gravity = Gravity.CENTER
    var llParam = LinearLayout.LayoutParams(
        LinearLayout.LayoutParams.WRAP_CONTENT,
        LinearLayout.LayoutParams.WRAP_CONTENT)
    llParam.gravity = Gravity.CENTER
    ll.layoutParams = llParam

    val progressBar = ProgressBar(context)
    progressBar.isIndeterminate = true
    progressBar.setPadding(0, 0, llPadding, 0)
    progressBar.layoutParams = llParam
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
        progressBar.indeterminateTintList = ColorStateList.valueOf(ContextCompat.getColor(context, progressColor))
    }

    llParam = LinearLayout.LayoutParams(
        ViewGroup.LayoutParams.WRAP_CONTENT,
        ViewGroup.LayoutParams.WRAP_CONTENT)
    llParam.gravity = Gravity.CENTER
    val tvText = TextView(context)
    tvText.text = message
    tvText.setTextColor(Color.parseColor("#000000"))
    tvText.textSize = 20.toFloat()
    tvText.layoutParams = llParam

    ll.addView(progressBar)
    ll.addView(tvText)

    val builder = AlertDialog.Builder(context)
    builder.setCancelable(true)
    builder.setView(ll)

    val dialog = builder.create()
    val window = dialog.window
    if (window != null) {
        val layoutParams = WindowManager.LayoutParams()
        layoutParams.copyFrom(dialog.window?.attributes)
        layoutParams.width = LinearLayout.LayoutParams.WRAP_CONTENT
        layoutParams.height = LinearLayout.LayoutParams.WRAP_CONTENT
        dialog.window?.attributes = layoutParams
    }
    return dialog
}

fun sendFeedbackEmail(context: Context, subject: String, body: String, emailClientChooserTitle: String) {

    val email = Intent(Intent.ACTION_SENDTO, Uri.parse("mailto:jbuhler@uoc.edu"))
    email.putExtra(Intent.EXTRA_SUBJECT, subject)
    email.putExtra(Intent.EXTRA_TEXT, body)

    startActivity(context, Intent.createChooser(email, emailClientChooserTitle), null)
}

// extension function to hide soft keyboard programmatically
fun Activity.hideSoftKeyboard(view: View) {
    (getSystemService(INPUT_METHOD_SERVICE) as InputMethodManager).apply {
        hideSoftInputFromWindow(view.windowToken, 0)
    }
}