/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.home

import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.annotations.NonNull
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.observers.DisposableObserver
import io.reactivex.schedulers.Schedulers

class HomePresenter(private var viewInterface: HomeContract.ViewInterface, private var remoteDataSource: RemoteDataSource) : HomeContract.PresenterInterface {

    private val compositeDisposable = CompositeDisposable()

    /// CALL RESULTS ///
    private val cacheIndexesObservable: Observable<Boolean> get() = remoteDataSource.cacheIndexes()
    private val cacheIndexesObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(loaded: Boolean) {
            }
            override fun onError(@NonNull e: Throwable) {
            }
            override fun onComplete() {
                //
            }
        }

    private val logoutObservable: Observable<Boolean> get() = remoteDataSource.logout()
    private val logoutObserver: DisposableObserver<Boolean>
        get() = object : DisposableObserver<Boolean>() {

            override fun onNext(loggedOut: Boolean) {
                if (loggedOut) {
                    viewInterface.loggedOut()
                } else {
                    viewInterface.logoutFailed()
                }
            }
            override fun onError(@NonNull e: Throwable) {
            }
            override fun onComplete() {
                //
            }
        }

    /// CALLS ///
    override fun cacheIndexes() {
        val cacheIndexesDisposable = cacheIndexesObservable
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(cacheIndexesObserver)

        compositeDisposable.add(cacheIndexesDisposable)
    }

    override fun getCurrentUserEmail(): String {
        return remoteDataSource.getCurrentUserEmail()
    }

    override fun logout() {
        val logoutDisposable = logoutObservable
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())
            .subscribeWith(logoutObserver)

        compositeDisposable.add(logoutDisposable)
    }

    override fun stop() {
        compositeDisposable.clear()
    }

}