/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.login

import android.app.AlertDialog
import android.app.Dialog
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.jbo.vga.R
import com.jbo.vga.extensions.hideSoftKeyboard
import com.jbo.vga.extensions.setProgressDialog
import com.jbo.vga.home.HomeActivity
import com.jbo.vga.model.RemoteDataSource
import com.jbo.vga.registration.RegistrationActivity
import kotlinx.android.synthetic.main.activity_login.*


class LoginActivity : AppCompatActivity(), LoginContract.ViewInterface {

    private lateinit var loginPresenter: LoginContract.PresenterInterface
    private lateinit var remoteDataSource: RemoteDataSource
    private lateinit var dialog: Dialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_login)

        initPresenter()

        stillAuthenticated()

        dialog = setProgressDialog(this, R.color.colorPrimary, getString(R.string.loading))

        // Set edit text focus change listener
        clLogin.onFocusChangeListener = View.OnFocusChangeListener { it, focused ->
            if (focused) {
                // Hide soft keyboard when edit text lost focus
                hideSoftKeyboard(it)
            }
        }

        // Register
        tvRegister.setOnClickListener {
            val intent = Intent(this, RegistrationActivity::class.java)
            startActivity(intent)
            overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
        }

        // Login
        buLogin.setOnClickListener {
            (loginPresenter as LoginPresenter).login(
                etLoginEmailAddress.text.toString(),
                etLoginPassword.text.toString()
            )
        }

        // Send password reset email
        tvforgottenPassword.setOnClickListener {
            loginPresenter.sendPasswordResetEmail(etLoginEmailAddress.text.toString())
        }
    }

    private fun initPresenter() {
        remoteDataSource = RemoteDataSource()
        loginPresenter = LoginPresenter(this, remoteDataSource)
    }

    private fun stillAuthenticated() {
        if (loginPresenter.stillAuthenticated()) {
            val intent = Intent(this, HomeActivity::class.java)
            startActivity(intent)
            overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
            finish()
        }
    }

    override fun showProgress() {
        dialog.show()
    }

    override fun hideProgress() {
        dialog.dismiss()
    }

    override fun loginSuccessfully() {
        val intent = Intent(this, HomeActivity::class.java)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
    }

    override fun passwordResetEmailSendSuccessfully() {
        val info = AlertDialog.Builder(this)
        info.setMessage(resources.getText(R.string.reset_password_email_send))
        info.setPositiveButton("Ok") { _, _ -> }
        info.show()
    }

    override fun displayError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
    }

    override fun onStop() {
        super.onStop()
        loginPresenter.stop()
    }

}