/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.audiocontrol

import android.content.Context
import android.media.MediaPlayer
import android.net.Uri
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import kotlinx.android.synthetic.main.fragment_audio_control.*
import kotlinx.android.synthetic.main.fragment_audio_control.view.*

class AudioControlFragment: Fragment() {

    private lateinit var level: String
    private var levelColor = R.color.colorDarkGray // default

    private lateinit var fragmentView: View
    private lateinit var mpSingle: MediaPlayer
    private lateinit var audioPathUri: Uri
    private var mpIsPaused = false


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            levelColor = levelColor(level)
            audioPathUri = it.getParcelable(ARG_AUDIO_PATH)!!
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        fragmentView = inflater.inflate(R.layout.fragment_audio_control, container, false)

        fragmentView.buPlay.setBackgroundColor(ContextCompat.getColor(fragmentView.context, levelColor))
        fragmentView.buPause.setBackgroundColor(ContextCompat.getColor(fragmentView.context, R.color.colorDarkGray))
        fragmentView.buStop.setBackgroundColor(ContextCompat.getColor(fragmentView.context, R.color.colorDarkGray))

        return fragmentView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        initMediaPlayer(view.context)

        view.buPlay.setOnClickListener {
            mpPlay(it.context)
        }

        view.buPause.setOnClickListener {
            mpPause(it.context)
        }

        view.buStop.setOnClickListener {
            mpStop(it.context)
        }
    }

    private fun initMediaPlayer(context: Context) {

        mpSingle = MediaPlayer.create(context, audioPathUri)

        mpSingle.setOnCompletionListener {
            mpSingle = MediaPlayer.create(context, audioPathUri)
            mpStop(context)
        }
    }


    private fun mpPlay(context: Context) {
        if (!mpIsPaused) {
            initMediaPlayer(context)
        }
        mpSingle.start()
        buPlay.isEnabled = false
        buPlay.setBackgroundColor(ContextCompat.getColor(context, R.color.colorDarkGray))
        buPause.isEnabled = true
        buPause.setBackgroundColor(ContextCompat.getColor(context, levelColor))
        buStop.isEnabled = true
        buStop.setBackgroundColor(ContextCompat.getColor(context, levelColor))
    }

    private fun mpPause(context: Context) {
        if (mpSingle.isPlaying) {
            mpSingle.pause()
        }
        mpIsPaused = true
        buPlay.isEnabled = true
        buPlay.setBackgroundColor(ContextCompat.getColor(context, levelColor))
        buPause.isEnabled = false
        buPause.setBackgroundColor(ContextCompat.getColor(context, R.color.colorDarkGray))
        buStop.isEnabled = true
        buStop.setBackgroundColor(ContextCompat.getColor(context, levelColor))
    }

    private fun mpStop(context: Context) {
        mpSingle.stop()
        mpIsPaused = false
        buPlay.isEnabled = true
        buPlay.setBackgroundColor(ContextCompat.getColor(context, levelColor))
        buPause.isEnabled = false
        buPause.setBackgroundColor(ContextCompat.getColor(context, R.color.colorDarkGray))
        buStop.isEnabled = false
        buStop.setBackgroundColor(ContextCompat.getColor(context, R.color.colorDarkGray))
    }

    override fun onPause() {
        super.onPause()
        mpPause(fragmentView.context)
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mpSingle.release()
    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_AUDIO_PATH = "audio_path"

        @JvmStatic
        fun newInstance(level: String, audioPathUri: Uri) =
            AudioControlFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putParcelable(ARG_AUDIO_PATH, audioPathUri)

                }
            }
    }

}