/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.listenstaticoptions

import android.media.MediaPlayer
import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageButton
import android.widget.LinearLayout
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.model.Content
import com.jbo.vga.model.Page
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_listen_static_options.view.*
import kotlin.math.roundToInt

class ListenStaticOptionsFragment: Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }

    private lateinit var level: String
    private lateinit var page: Page
    private lateinit var content : Content

    private lateinit var mpSingle: MediaPlayer

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            page = it.getSerializable(ARG_PAGE) as Page
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        return inflater.inflate(R.layout.fragment_listen_static_options, container, false)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        content = page.contents.first()

        pageCallback.addAudioFragment(level, content.audioPath)

        createOptions(view, content)
    }

    private fun createOptions(view: View, content: Content) {

        val actionListener = View.OnClickListener{ selectedOption ->

            // Play sound
            val option = content.options[selectedOption.id - 1]
            if (this::mpSingle.isInitialized) {
                mpSingle.release()
            }
            mpSingle = MediaPlayer.create(view.context, pageCallback.getAudioUri(option.audioPath))
            mpSingle.start()
        }

        // Static option width
        val optionWidth = (resources.displayMetrics.widthPixels * 0.18).roundToInt()

        var linearLayout: LinearLayout ?= null
        for ((index, option) in content.options.withIndex()) {

            //  buttons per row
            val newRow = (index % BUTTONS_PER_ROW == 0)
            if (newRow) {
                // Create new row
                linearLayout = LinearLayout(view.context)
                val layoutParamsRow = LinearLayout.LayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT))
                layoutParamsRow.weight = 1F
                linearLayout.layoutParams = layoutParamsRow
                linearLayout.gravity = Gravity.CENTER_HORIZONTAL
                linearLayout.orientation = LinearLayout.HORIZONTAL
            }

            // Inflate option button used to create the options dynamically
            val ibOption =  LayoutInflater.from(view.context).inflate(R.layout.image_button_option, null, false) as ImageButton
            ibOption.id = index + 1 // Adaptation of 0 beginning

            val layoutParamsOption = LinearLayout.LayoutParams(LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT))
            layoutParamsOption.width = optionWidth
            ibOption.layoutParams = layoutParamsOption

            val pathUri= pageCallback.getImageUri(option.imagePath)
            pageCallback.loadSvgImage(pathUri, ibOption)
            ibOption.setOnClickListener(actionListener)

            linearLayout?.addView(ibOption)

            // Completed row -> add to main view
            if (newRow) {
                view.llStaticOptions.addView(linearLayout)
            }
        }

    }

    override fun onDestroyView() {
        super.onDestroyView()
        if (this::mpSingle.isInitialized) {
            mpSingle.release()
        }
    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_PAGE = "page"

        const val BUTTONS_PER_ROW = 5

        @JvmStatic
        fun newInstance(level: String, page: Page) =
            ListenStaticOptionsFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putSerializable(ARG_PAGE, page)

                }
            }
    }

}