/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.readingcomprehension

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.text.method.ScrollingMovementMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.model.Content
import com.jbo.vga.model.Page
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_reading_comprehension.*
import kotlinx.android.synthetic.main.fragment_reading_comprehension.view.*


class ReadingComprehensionFragment: Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }

    private lateinit var level: String
    private lateinit var page: Page
    private lateinit var currentContent : Content
    private var currentContentIndex = -1


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            page = it.getSerializable(ARG_PAGE) as Page
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val fragmentView = inflater.inflate(R.layout.fragment_reading_comprehension, container, false)

        fragmentView.tvText.movementMethod = ScrollingMovementMethod()
        return fragmentView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        val actionListener = View.OnClickListener{ selectedOption ->

            if (selectedOption.tag == currentContent.correctOptionSequence) {
                selectedOption.setBackgroundColor(ContextCompat.getColor(view.context, R.color.correctAnswerColor))
            } else {
                selectedOption.setBackgroundColor(ContextCompat.getColor(view.context, R.color.incorrectAnswerColor))
            }
            nextContent(view, false)
        }

        buTrue.setOnClickListener(actionListener)
        buFalse.setOnClickListener(actionListener)

        // The first content corresponds to the text
        currentContentIndex++
        currentContent = page.contents[currentContentIndex]
        view.tvText.text = currentContent.de

        // Questions ...
        nextContent(view, true)
    }

    private fun nextContent(view: View, firstQuestion: Boolean) {

        // Disable options to avoid user actions until next content is loaded
        view.buTrue.isEnabled = false
        view.buFalse.isEnabled = false

        val timeUntilNextContent = if (firstQuestion) 0 else TIME_UNTIL_NEXT_CONTENT_ML
        Handler(Looper.getMainLooper()).postDelayed({

            view.buTrue.setBackgroundColor(ContextCompat.getColor(view.context, levelColor(level)))
            view.buFalse.setBackgroundColor(ContextCompat.getColor(view.context, levelColor(level)))
            currentContentIndex++

            // Finished?
            if (currentContentIndex < page.contents.size) {

                // No: load next content
                currentContent = page.contents[currentContentIndex]
                view.tvContent?.text = currentContent.de

                // Enable options to avoid user actions until next content is loaded
                view.buTrue.isEnabled = true
                view.buFalse.isEnabled = true

            } else {
                // Return to index
                pageCallback.onContentFinished()
            }

        }, timeUntilNextContent)

    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_PAGE = "page"

        const val TIME_UNTIL_NEXT_CONTENT_ML: Long = 1500

        @JvmStatic
        fun newInstance(level: String, page: Page) =
             ReadingComprehensionFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putSerializable(ARG_PAGE, page)

                }
            }
    }

}