/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.textwriting

import android.app.AlertDialog
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.extensions.levelColor
import com.jbo.vga.extensions.sendFeedbackEmail
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_text_writing.*
import kotlinx.android.synthetic.main.fragment_text_writing.view.*


class TextWritingFragment : Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }

    private lateinit var level: String

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        // Inflate the layout for this fragment
        val fragmentView = inflater.inflate(R.layout.fragment_text_writing, container, false)

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            fragmentView.etText.background.setTint(levelColor(level))
        }

        fragmentView.buFeedback.setBackgroundColor(ContextCompat.getColor(fragmentView.context, levelColor(level)))

        return fragmentView
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        // Set focus change listener
        clTextWriting.onFocusChangeListener = View.OnFocusChangeListener { it, focused ->
            if (focused) {
                // Hide soft keyboard when edit text lost focus
                pageCallback.onHideSoftKeyboard(it)
            }
        }

        buFeedback.setOnClickListener {

            if (etText.length() > 0) {
                sendFeedbackEmail(view.context, pageCallback.getFeedbackEmailSubject(level), etText.text.toString(), resources.getText(R.string.email_client).toString())
            } else {
                val info = AlertDialog.Builder(view.context)
                info.setMessage(resources.getText(R.string.empty_text))
                info.setPositiveButton("Ok") { _, _ ->}
                info.show()
            }
        }
    }


    companion object {

        private const val ARG_LEVEL = "level"

        @JvmStatic
        fun newInstance(level: String) =
            TextWritingFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)

                }
            }
    }

}