/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.page.fragments.theory

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import com.jbo.vga.R
import com.jbo.vga.model.Page
import com.jbo.vga.page.PageListener
import kotlinx.android.synthetic.main.fragment_theory_image.view.*

class TheoryImageFragment : Fragment() {

    // Page interface
    private lateinit var pageCallback: PageListener
    fun setPageListener(pageCallback: PageListener) {
        this.pageCallback = pageCallback
    }


    private lateinit var level: String
    private lateinit var page: Page

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        arguments?.let {
            level = it.getString(ARG_LEVEL).toString()
            page = it.getSerializable(ARG_PAGE) as Page
        }

    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        // Inflate the layout for this fragment
        val fragmentView = inflater.inflate(R.layout.fragment_theory_image, container, false)

        // Load image
        val pathUri= pageCallback.getImageUri(page.contents.first().imagePath)
        pageCallback.loadSvgImage(pathUri, fragmentView.pvTheory)

        return fragmentView
    }

    companion object {

        private const val ARG_LEVEL = "level"
        private const val ARG_PAGE = "page"

        @JvmStatic
        fun newInstance(level: String, page: Page) =
            TheoryImageFragment().apply {
                arguments = Bundle().apply {

                    putString(ARG_LEVEL, level)
                    putSerializable(ARG_PAGE, page)

                }
            }
    }
}