/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.registration

import android.app.Dialog
import android.content.Intent
import android.os.Bundle
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import com.jbo.vga.R
import com.jbo.vga.extensions.hideSoftKeyboard
import com.jbo.vga.extensions.setProgressDialog
import com.jbo.vga.home.HomeActivity
import com.jbo.vga.model.RemoteDataSource
import kotlinx.android.synthetic.main.activity_registration.*

class RegistrationActivity : AppCompatActivity(), RegistrationContract.ViewInterface {

    private lateinit var registrationPresenter: RegistrationContract.PresenterInterface
    private lateinit var dialog: Dialog

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_registration)

        initPresenter()

        initToolbar()

        dialog = setProgressDialog(this, R.color.colorPrimary, getString(R.string.loading))

        // Set edit text focus change listener
        clRegistration.onFocusChangeListener = View.OnFocusChangeListener { it, focused ->
            if (focused) {
                // Hide soft keyboard when edit text lost focus
                hideSoftKeyboard(it)
            }
        }

        buCreateAccount.setOnClickListener {
            (registrationPresenter as RegistrationPresenter).createAccount(
                etRegistrationEmailAddress.text.toString(),
                etRegistrationPassword.text.toString()
            )
        }
    }

    private fun initPresenter() {
        val remoteDataSource = RemoteDataSource()
        registrationPresenter = RegistrationPresenter(this, remoteDataSource)
    }

    private fun initToolbar() {
        setSupportActionBar(tbRegistration)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        return when (item.itemId) {
            android.R.id.home -> {
                onBackPressed()
                return true
            }
            else -> false
        }
    }

    override fun showProgress() {
        dialog.show()
    }

    override fun hideProgress() {
        dialog.dismiss()
    }

    override fun registeredSuccessfully() {
        val intent = Intent(this, HomeActivity::class.java)
        startActivity(intent)
        overridePendingTransition(R.anim.slide_in_left, R.anim.slide_out_right)
    }

    override fun displayError(message: String) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show()
    }

    override fun finish() {
        super.finish()
        overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left)
    }

    override fun onStop() {
        super.onStop()
        registrationPresenter.stop()
    }

}