/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.credits

import com.jbo.vga.BaseTest
import com.jbo.vga.RxImmediateSchedulerRule
import com.jbo.vga.model.Credit
import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import org.junit.Before
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith
import org.mockito.Mock
import org.mockito.Mockito
import org.mockito.junit.MockitoJUnitRunner

@RunWith(MockitoJUnitRunner::class)
class CreditsTests: BaseTest() {

    @Rule
    @JvmField
    var testSchedulerRule = RxImmediateSchedulerRule()

    @Mock
    private lateinit var mockActivity: CreditsContract.ViewInterface

    @Mock
    private lateinit var mockRemoteDataSource: RemoteDataSource

    private lateinit var creditsPresenter: CreditsPresenter

    @Before
    fun setUp() {
        creditsPresenter = CreditsPresenter(
            viewInterface = mockActivity,
            remoteDataSource = mockRemoteDataSource
        )
    }

    @Test
    fun testGetCreditsList() {
        //Set up
        val remoteCredits = dummyCredits
        Mockito.doReturn(Observable.just(remoteCredits)).`when`(mockRemoteDataSource).getCredits()

        //Invoke
        creditsPresenter.getCredits()

        //Assert
        Mockito.verify(mockRemoteDataSource).getCredits()
        Mockito.verify(mockActivity).displayCredits(remoteCredits)
    }

    @Test
    fun testGetEmptyCreditsList() {
        //Set up
        val emptyCreditsList = emptyList<Credit>()
        Mockito.doReturn(Observable.just(ArrayList<Credit>())).`when`(mockRemoteDataSource).getCredits()

        //Invoke
        creditsPresenter.getCredits()

        //Assert
        Mockito.verify(mockRemoteDataSource).getCredits()
        Mockito.verify(mockActivity).displayCredits(emptyCreditsList)
    }


    //
    // Helper functions
    //
    private val dummyCredits: ArrayList<Credit>
        get() {
            val dummyCredits = ArrayList<Credit>()
            dummyCredits.add(
                Credit(
                    "Credit title 01",
                    "Credit link 01",
                )
            )
            dummyCredits.add(
                Credit(
                    "Credit title 02",
                    "Credit link 02",
                )
            )
            dummyCredits.add(
                Credit(
                    "Credit title 03",
                    "Credit link 03",
                )
            )
            dummyCredits.add(
                Credit(
                    "Credit title 04",
                    "Credit link 04",
                )
            )
            dummyCredits.add(
                Credit(
                    "Credit title 05",
                    "Credit link 05",
                )
            )

            return dummyCredits
        }
}