/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.home

import com.jbo.vga.BaseTest
import com.jbo.vga.RxImmediateSchedulerRule
import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import org.junit.Before
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith
import org.mockito.Mock
import org.mockito.Mockito
import org.mockito.junit.MockitoJUnitRunner

@RunWith(MockitoJUnitRunner::class)
class HomeTests: BaseTest() {

    @Rule
    @JvmField
    var testSchedulerRule = RxImmediateSchedulerRule()

    @Mock
    private lateinit var mockActivity: HomeContract.ViewInterface

    @Mock
    private lateinit var mockRemoteDataSource: RemoteDataSource

    private lateinit var homePresenter: HomePresenter
    private val email = "current@user.es"

    @Before
    fun setUp() {
        homePresenter = HomePresenter(
            viewInterface = mockActivity,
            remoteDataSource = mockRemoteDataSource
        )
    }

    @Test
    fun testCacheIndexes() {
        //Set up
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteDataSource).cacheIndexes()

        //Invoke
        homePresenter.cacheIndexes()

        //Assert
        Mockito.verify(mockRemoteDataSource).cacheIndexes()
    }

    @Test
    fun testGetCurrentUserEmail() {
        //Set up
        Mockito.doReturn(email).`when`(mockRemoteDataSource).getCurrentUserEmail()

        //Invoke
        homePresenter.getCurrentUserEmail()

        //Assert
        Mockito.verify(mockRemoteDataSource).getCurrentUserEmail()
    }

    @Test
    fun testLogoutSuccessfully() {
        //Set up
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteDataSource).logout()

        //Invoke
        homePresenter.logout()

        //Assert
        Mockito.verify(mockRemoteDataSource).logout()
        Mockito.verify(mockActivity).loggedOut()
    }

    @Test
    fun testLogoutFailed() {
        //Set up
        Mockito.doReturn(Observable.just(false)).`when`(mockRemoteDataSource).logout()

        //Invoke
        homePresenter.logout()

        //Assert
        Mockito.verify(mockRemoteDataSource).logout()
        Mockito.verify(mockActivity).logoutFailed()
    }

}