/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.index

import com.jbo.vga.BaseTest
import com.jbo.vga.RxImmediateSchedulerRule
import com.jbo.vga.model.DidacticUnit
import com.jbo.vga.model.RemoteDataSource
import io.reactivex.Observable
import org.junit.Before
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith
import org.mockito.Mock
import org.mockito.Mockito
import org.mockito.junit.MockitoJUnitRunner

@RunWith(MockitoJUnitRunner::class)
class IndexTests: BaseTest() {

    @Rule
    @JvmField
    var testSchedulerRule = RxImmediateSchedulerRule()

    @Mock
    private lateinit var mockActivity: IndexContract.ViewInterface

    @Mock
    private lateinit var mockRemoteDataSource: RemoteDataSource

    private lateinit var indexPresenter: IndexPresenter
    private lateinit var level: String

    @Before
    fun setUp() {
        indexPresenter = IndexPresenter(
            viewInterface = mockActivity,
            remoteDataSource = mockRemoteDataSource
        )
        level = "a1"
    }

    @Test
    fun testGetLevelIndex() {
        //Set up
        val remoteIndex = dummyIndex
        Mockito.doReturn(Observable.just(remoteIndex)).`when`(mockRemoteDataSource).getLevelIndex(level)

        //Invoke
        indexPresenter.getLevelIndex(level)

        //Assert
        Mockito.verify(mockRemoteDataSource).getLevelIndex(level)
        Mockito.verify(mockActivity).displayIndex(remoteIndex)
    }

    @Test
    fun testGetEmptyIndex() {
        //Set up
        val emptyIndex = emptyList<DidacticUnit>()
        Mockito.doReturn(Observable.just(ArrayList<DidacticUnit>())).`when`(mockRemoteDataSource).getLevelIndex(level)

        //Invoke
        indexPresenter.getLevelIndex(level)

        //Assert
        Mockito.verify(mockRemoteDataSource).getLevelIndex(level)
        Mockito.verify(mockActivity).displayIndex(emptyIndex)
    }


    //
    // Helper functions
    //
    private val dummyIndex: ArrayList<DidacticUnit>
        get() {
            val dummyIndex = ArrayList<DidacticUnit>()
            dummyIndex.add(
                DidacticUnit(
                    "Title 01 de",
                    "Title 01 es",
                    "published",
                    "",
                    "yes"
                )
            )
            dummyIndex.add(
                DidacticUnit(
                    "Title 02 de",
                    "Title 02 es",
                    "published",
                    "",
                    "yes"
                )
            )
            dummyIndex.add(
                DidacticUnit(
                    "Title 03 de",
                    "Title 03 es",
                    "published",
                    "",
                    "yes"
                )
            )
            dummyIndex.add(
                DidacticUnit(
                    "Title 04 de",
                    "Title 04 es",
                    "published",
                    "",
                    "yes"
                )
            )
            dummyIndex.add(
                DidacticUnit(
                    "Title 05 de",
                    "Title 05 es",
                    "published",
                    "",
                    "yes"
                )
            )

            return dummyIndex
        }
}