/*
 Copyright 2020 Jan Bühler Olivé

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
*/
package com.jbo.vga.login

import com.jbo.vga.BaseTest
import com.jbo.vga.RxImmediateSchedulerRule
import com.jbo.vga.model.RemoteDataSource
import com.nhaarman.mockito_kotlin.never
import io.reactivex.Observable
import org.junit.Before
import org.junit.Rule
import org.junit.Test
import org.junit.runner.RunWith
import org.mockito.Mock
import org.mockito.Mockito
import org.mockito.junit.MockitoJUnitRunner

@RunWith(MockitoJUnitRunner::class)
class LoginTests: BaseTest() {

    @Rule
    @JvmField
    var testSchedulerRule = RxImmediateSchedulerRule()

    @Mock
    private lateinit var mockActivity: LoginContract.ViewInterface

    @Mock
    private lateinit var mockRemoteDataSource: RemoteDataSource

    private lateinit var loginPresenter: LoginPresenter
    private val email = "test@test.es"
    private val password = "password"

    @Before
    fun setUp() {
        loginPresenter = LoginPresenter(
            viewInterface = mockActivity,
            remoteDataSource = mockRemoteDataSource
        )
    }

    @Test
    fun testLoginSuccessfully() {
        //Set up
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteDataSource).login(email, password)

        //Invoke
        loginPresenter.login(email, password)

        //Assert
        Mockito.verify(mockRemoteDataSource).login(email, password)
        Mockito.verify(mockActivity).loginSuccessfully()
    }

    @Test
    fun testLoginFailed() {
        //Set up
        Mockito.doReturn(Observable.just(false)).`when`(mockRemoteDataSource).login(email, password)

        //Invoke
        loginPresenter.login(email, password)

        //Assert
        Mockito.verify(mockRemoteDataSource).login(email, password)
        Mockito.verify(mockActivity, never()).loginSuccessfully()
    }

    @Test
    fun testSendPasswordResetEmail() {
        //Set up
        Mockito.doReturn(Observable.just(true)).`when`(mockRemoteDataSource).sendPasswordResetEmail(email)

        //Invoke
        loginPresenter.sendPasswordResetEmail(email)

        //Assert
        Mockito.verify(mockRemoteDataSource).sendPasswordResetEmail(email)
        Mockito.verify(mockActivity).passwordResetEmailSendSuccessfully()
    }

    @Test
    fun testStillAuthenticated() {
        //Set up
        Mockito.doReturn(true).`when`(mockRemoteDataSource).stillAuthenticated()

        //Invoke
        loginPresenter.stillAuthenticated()

        //Assert
        Mockito.verify(mockRemoteDataSource).stillAuthenticated()
    }

}